diff --git a/usr.bin/bc/Makefile b/usr.bin/bc/Makefile
index 7ea6bb3..d6ef15b 100644
--- a/usr.bin/bc/Makefile
+++ b/usr.bin/bc/Makefile
@@ -6,6 +6,9 @@ SRCS=		bc.y scan.l y.tab.h
 CFLAGS+=	-I. -I${.CURDIR}
 WARNS?=		2
 
+DPADD=	${LIBEDIT} ${LIBTERMCAP}
+LDADD=	-ledit -ltermcap
+
 SCRIPTS=	bc.library
 SCRIPTSMODE=	444
 SCRIPTSDIR=	/usr/share/misc
diff --git a/usr.bin/bc/bc.y b/usr.bin/bc/bc.y
index 1f25152..9c89555 100644
--- a/usr.bin/bc/bc.y
+++ b/usr.bin/bc/bc.y
@@ -1,7 +1,6 @@
 %{
 /*
  * $OpenBSD: bc.y,v 1.32 2006/05/18 05:49:53 otto Exp $
- * $DragonFly: src/usr.bin/bc/bc.y,v 1.3 2007/09/01 18:42:08 pavalos Exp $
  */
 
 /*
@@ -39,6 +38,7 @@
 #include <ctype.h>
 #include <err.h>
 #include <errno.h>
+#include <histedit.h>
 #include <limits.h>
 #include <search.h>
 #include <signal.h>
@@ -1076,6 +1076,13 @@ sigchld(int signo)
 	}
 }
 
+static const char *
+dummy_prompt(void)
+{
+
+        return ("");
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -1132,6 +1139,16 @@ main(int argc, char *argv[])
 			dup(p[1]);
 			close(p[0]);
 			close(p[1]);
+			if (interactive) {
+				el = el_init("bc", stdin, stderr, stderr);
+				hist = history_init();
+				history(hist, &he, H_SETSIZE, 100);
+				el_set(el, EL_HIST, history, hist);
+				el_set(el, EL_EDITOR, "emacs");
+				el_set(el, EL_SIGNAL, 1);
+				el_set(el, EL_PROMPT, dummy_prompt);
+				el_source(el, NULL);
+			}
 		} else {
 			close(STDIN_FILENO);
 			dup(p[0]);
diff --git a/usr.bin/bc/extern.h b/usr.bin/bc/extern.h
index 0ba989b..2113361 100644
--- a/usr.bin/bc/extern.h
+++ b/usr.bin/bc/extern.h
@@ -39,4 +39,7 @@ extern int	sargc;
 extern char	**sargv;
 extern char	*filename;
 extern char	*cmdexpr;
-bool		interactive;
+extern bool		 interactive;
+extern EditLine		*el;
+extern History		*hist;
+extern HistEvent	 he;
diff --git a/usr.bin/bc/scan.l b/usr.bin/bc/scan.l
index 5bc1a17..c11b580 100644
--- a/usr.bin/bc/scan.l
+++ b/usr.bin/bc/scan.l
@@ -22,6 +22,7 @@
 
 #include <err.h>
 #include <errno.h>
+#include <histedit.h>
 #include <signal.h>
 #include <stdbool.h>
 #include <string.h>
@@ -36,12 +37,21 @@
 int		lineno;
 bool		interactive;
 
+HistEvent	 he;
+EditLine	*el;
+History		*hist;
+
 static char	*strbuf = NULL;
 static size_t	strbuf_sz = 1;
 static bool	dot_seen;
 
 static void	init_strbuf(void);
 static void	add_str(const char *);
+static int	 bc_yyinput(char *, int);
+ 
+#undef YY_INPUT
+#define YY_INPUT(buf,retval,max) \
+	(retval = bc_yyinput(buf, max))
 
 %}
 
@@ -285,3 +295,31 @@ yywrap(void)
 	}
 	return (1);
 }
+
+static int
+bc_yyinput(char *buf, int maxlen)
+{
+	int num;
+	if (yyin == stdin && interactive) {
+		const char *bp;
+
+		if ((bp = el_gets(el, &num)) == NULL || num == 0)
+			return (0);
+		if (num > maxlen) {
+			el_push(el, (char *)(uintptr_t)(bp) + maxlen);
+			num = maxlen;
+		}
+		memcpy(buf, bp, num);
+		history(hist, &he, H_ENTER, bp);
+	} else {
+		int c = '*';
+		for (num = 0; num < maxlen &&
+		    (c = getc(yyin)) != EOF && c != '\n'; ++num)
+			buf[num] = (char) c;
+		if (c == '\n')
+			buf[num++] = (char) c;
+		if (c == EOF && ferror(yyin))
+			YY_FATAL_ERROR( "input in flex scanner failed" );
+	}
+	return (num);
+}
